/******************************************************************************/
// Real time program ceoxTask1 implementation file
// ceoxTask1.cpp
/******************************************************************************/
#include "ceoxTask1.h"
/******************************************************************************/
#pragma pack(push, SORCPACK, 4) //struct members DWORD aligned
/******************************************************************************/
// PDT structure
typedef struct _PDT_Type
{
   MAX_PDT_HEAD    rcHead;
   DWORD main_proc;
   DWORD auto_init;
   // any number of user defined procedures may follow (optionally):
   //%NEW_PROC_PDT%  !!! do not delete this line !!!
   
} PDT_Type, *PPDT_Type;


// parameter range structure
typedef struct _Parameter_Type
{
   MAX_TDT_TYPE rcTDT;           // always before the first parameter !
   // offset 0

   MAX_ERROR   Error;            // offset 0 Errorcode
   MAXMODHND   hMyself;          // offset 4 Handle for the CPU module
   MAX_OS_INFO rcOsX;            // offset 8 Version info about OsX
   
   ULONG       ulLedLimit;       // offset 24
   ULONG       ulCounter;        // offset 28
	DWORD       dwLedState;       // offset 32
	MAXMODHND   hHost;            // offset 36 handle for the SRQ Host
   ushort      usSrq;            // offset 40 srq to send on LED switch
} Parameter_Type, *PParameter_Type;

/******************************************************************************/
#pragma pack(pop, SORCPACK)   //restore struct alignment
/******************************************************************************/
//globals
PDT_Type       g_rcPDT;
Parameter_Type g_rcPar;
/******************************************************************************/
// main procedure of the task (called automatically by OsX after activating the task)
void MAXEXPORT MainProc(void)
{
   max_entry_proc();
   
   g_rcPar.ulCounter++;

   //DEBUGMSG (ZONE_INFO, (DTAG MSGTEXT("MainProc %8u\n"), g_rcPar.ulCounter));

   if(g_rcPar.ulCounter < g_rcPar.ulLedLimit)
   {
      max_exit_proc();
   }

   g_rcPar.ulCounter = 0;

   if(g_rcPar.dwLedState)
   {
      // LED-OFF
      g_rcPar.Error = max_led_off(g_rcPar.hMyself);

      LOGMSG (ZONE_INFO, (DTAG MSGTEXT("Led Off %04X\n"), g_rcPar.Error));

      g_rcPar.dwLedState = 0;
   }
   else
   {
      // LED-ON
      g_rcPar.Error = max_led_on(g_rcPar.hMyself);

      LOGMSG (ZONE_INFO, (DTAG MSGTEXT("Led On  %04X\n"), g_rcPar.Error));

      g_rcPar.dwLedState = 1;
   }

   g_rcPar.Error = max_send_srq(g_rcPar.hHost, 0, g_rcPar.usSrq);

   max_exit_proc();
}

/******************************************************************************/
// Init task parameter
void MAXEXPORT AutoInit(void)
{
   max_entry_proc();

   LOGMSG(ZONE_INFO, (DTAG MSGTEXT("AutoInit\n")));

   max_init_lib(0);
   // get a handle for the X-MAX-400
   g_rcPar.Error = max_connect_cpu (MAX_LOCAL_BOARD, MAX_MYSELF, 0,&g_rcPar.rcOsX, &g_rcPar.hMyself);

   g_rcPar.ulLedLimit = 100000;
	g_rcPar.ulCounter = 0;

	//Connect to SRQ Host
	g_rcPar.Error = max_connect_cpu (MAX_WINCE_HOST, 0, 0, NULL, &g_rcPar.hHost);
   g_rcPar.usSrq = 0x1234;

   max_exit_proc();
}

//%NEW_PROC_IMPLEMENT%  !!! do not delete this line !!!
/******************************************************************************/
/******************************************************************************/
// initialize the Program Descriptor Table (PDT)
void main ()
{
   // the PDT-flags determine the task-type and
   // information how the data and parameter range are allocated
   
   //set flags for task type
   g_rcPDT.rcHead.usFlags     = MAX_NI_TASK | 
                                MAXPDT_TASKTYP_IN_PDT |
                                MAXPDT_FIXED_DATASIZE |
                                MAXPDT_DATA_IN_PROG |
                                MAXPDT_PARAMETER_IN_PROG;

   // interrupt source
	g_rcPDT.rcHead.usInterrupt = INTERRUPT_NUMBER;
   
   // the program number identifies the program uniquely
   g_rcPDT.rcHead.usProgNo    = PROGRAMM_NUMBER;
   
   // number of global task procedures in the PDT
   g_rcPDT.rcHead.usProcedures = (sizeof(PDT_Type) - sizeof (MAX_PDT_HEAD)) / 4;
   
   // the following lines are just for documentation and information purpose
   g_rcPDT.rcHead.ucType          = 1;   // always !
   g_rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
   g_rcPDT.rcHead.cVersion        = '0' + PROGRAM_MAJOR_VERSION;
   g_rcPDT.rcHead.cRevision       = PROGRAM_MINOR_VERSION;
   g_rcPDT.rcHead.ucCPU           = MAX_CPUARMV4I;
   g_rcPDT.rcHead.ucCoProc        = _NOCOPROC;
   g_rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
   g_rcPDT.rcHead.ucProgType      = 0;   // not used
   g_rcPDT.rcHead.ulRes1          = 0;
   g_rcPDT.rcHead.ulRes2          = 0;
   g_rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet
   
   // address and size of the tasks data range
   // only when MAXPDT_DATA_IN_PROG is set in PDT-flags
   g_rcPDT.rcHead.ulDataAdr       = 0;
   g_rcPDT.rcHead.ulDataSize      = 0;
   
   // address and size of the tasks parameter range
   // only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
   g_rcPDT.rcHead.ulParAdr  = (ULONG)&g_rcPar + sizeof(MAX_TDT_TYPE);
   g_rcPDT.rcHead.usParSize = sizeof(Parameter_Type) - sizeof(MAX_TDT_TYPE);
   
   // initialize global procedure addresses in PDT
   g_rcPDT.main_proc        = (DWORD)MainProc;
   g_rcPDT.auto_init        = (DWORD)AutoInit;
   //ToDo: register user defined procedures:
   //%NEW_PROC_SET%  !!! do not delete this line !!!
   
   // transfer PDT information to the OsX
   max_init_program (&g_rcPDT.rcHead);
}
/******************************************************************************/
/******************************************************************************/
