/**
* Program to load and install debuggable CEoX tasks 
*/

#include "stdafx.h"
#include "max_lib.h"
#include "ceox_instprog_dbg.h"

#define CMDLINE_PARAMS 4

extern "C" {
   HANDLE LoadIntChainHandler(LPCTSTR lpszFileName, LPCTSTR lpszFunctionName, BYTE bIRQ);
   BOOL KernelLibIoControl(HANDLE hLib, DWORD dwIoControlCode, LPVOID lpInBuf, DWORD nInBufSize, LPVOID lpOutBuf, DWORD nOutBufSize, LPDWORD lpBytesReturned);
}

typedef struct
{
   USHORT   usProcNo;
   USHORT   usTaskNo;
   USHORT   usRetVal;
   USHORT   usInSize;
   USHORT   usOutSize;
} sExecProcParams;

HANDLE g_hCeoxIst = INVALID_HANDLE_VALUE;
HANDLE g_hCeoxIisr = INVALID_HANDLE_VALUE;
//*****************************************************************************
// GetCeoxHandle
/**
*/
//*****************************************************************************
HANDLE GetCeoxHandle()
{
   wchar_t	awcShortName[4];
   ULONG nBytesRead;
   BOOL bResult;

   wcscpy(awcShortName, L"CST1:");
   g_hCeoxIst = CreateFile (awcShortName, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_WRITE| FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
   if (g_hCeoxIst == INVALID_HANDLE_VALUE)
  		return INVALID_HANDLE_VALUE;

	//get handle from IST
	bResult = ReadFile(g_hCeoxIst, &g_hCeoxIisr, 4, &nBytesRead, NULL) ;
	
	if (bResult && (nBytesRead > 0))
      return g_hCeoxIisr;

   printf("Unable to get CEoX handle!\n");
   return INVALID_HANDLE_VALUE;
}
//*****************************************************************************
// LoadCeoxTask
/**
*/
//*****************************************************************************
HANDLE LoadCeoxTask(wchar_t *filename, USHORT usProgNo, USHORT usTaskNo)
{
   IST_TASK_PROG_NO_NAME rcIstTask;
   DWORD nBytesReturned;
   ULONG ulRetVal;

   if(g_hCeoxIst != INVALID_HANDLE_VALUE)
   {
      if(LoadIntChainHandler(filename, TEXT("ISRHandler"), 0) == NULL)
         return INVALID_HANDLE_VALUE;

      wcscpy(rcIstTask.awcProgName, filename);
      rcIstTask.usTaskNo = usTaskNo;
      //JS 11.02.14
      rcIstTask.usProgNo = usProgNo;
      
      //load via ceoxIST -> handle with kernelrights
      if(DeviceIoControl(g_hCeoxIst, IOCTL_LOAD_USER_TASK_IST, &rcIstTask, sizeof(rcIstTask), 
         &ulRetVal, sizeof(ULONG), &nBytesReturned, NULL) && (nBytesReturned == sizeof(ULONG)))
         return (HANDLE)ulRetVal;
   }
   return INVALID_HANDLE_VALUE;
}
//*****************************************************************************
// Main Prog
/**
*	
*						
*					
*/
//*****************************************************************************
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPTSTR lpCmdLine, int nCmdShow)
{
   char line[81];

   WCHAR *pwcStart;
   DWORD dwBytesReturned, dwInData;
   DWORD dwCMainAddress;
   ULONG ulProgNo, ulTaskNo;
   HANDLE hCeoxProg;
   BOOL bInstallTask;
   CEOX_INFO rcCeoxInfo;
   CEOX_INSTALL_TASK rcInstallTask;
   CEOX_INSTALL_TASK_OUT rcInstallTaskOut;

   sExecProcParams *pexecparams;
	
   WCHAR* pwcCmd[CMDLINE_PARAMS];
   int i;
   MAX_PDT_HEAD* prcPdtAddress;

  // check, if CEoX is running
	GetCeoxHandle();

   if (g_hCeoxIisr == INVALID_HANDLE_VALUE)
	{
      printf("CEoX not running!\n");
		return -1;
	}
   if (!KernelLibIoControl(g_hCeoxIisr, IOCTL_CEOX_GET_INFO, NULL, 0, &rcCeoxInfo, sizeof(rcCeoxInfo), &dwBytesReturned)
      || (rcCeoxInfo.dwInitDone == 0))
	{
      printf("CEoX not running!\n");
		return -1;
	}

   // check command line
   pwcStart = lpCmdLine;
   i=0;
   while((pwcStart != NULL) && (i < CMDLINE_PARAMS))
   {
      pwcCmd[i++] = pwcStart;
      pwcStart = wcschr(pwcStart, L' ');
      if(pwcStart == NULL)
      {
         if(i < CMDLINE_PARAMS)
            pwcCmd[i++] = pwcStart;
      }
      else
         *pwcStart++ = 0;
   }
   while(i < CMDLINE_PARAMS)
      pwcCmd[i++] = NULL;
   
   if(pwcCmd[2] == NULL)
   {
      printf("Usage: ceox_instprog_dbg \\path\\progname.dll ProgNo TaskNo\n");
      printf("ProgNo and TaskNo in Hex\n");
      return -2;
   }
   wprintf(L"Program name: %s\n", pwcCmd[0]);
   
   if((swscanf(pwcCmd[1], L"%x", &ulProgNo) != 1) || (ulProgNo == 0) || (ulProgNo > 0xFFFF))
   {
      printf("The program number has to be entered as hex number, e. g. A6F\n");
      return -3;
   }
   printf("Program number: 0x%04X\n", ulProgNo);
   
   if((swscanf(pwcCmd[2], L"%x", &ulTaskNo) != 1) || (ulTaskNo == 0) || (ulTaskNo > 1023))
   {
      printf("The task number has to be entered as hex number, e. g. 4E\n");
      printf("Valid task numbers are from 0001 to 03FF\n");
      return -4;
   }
   printf("Task number: 0x%04X\n", ulTaskNo);
   
   bInstallTask = ((pwcCmd[3] == NULL) || (_wcsicmp(pwcCmd[3], TEXT("-noinstall")) != 0));
      
// install program by LoadIntChainHandler (for debug only)
   hCeoxProg = LoadCeoxTask(pwcCmd[0], (USHORT)ulProgNo, (USHORT)ulTaskNo);
   if (hCeoxProg == INVALID_HANDLE_VALUE)
   {
      printf("Program could not be loaded!\n");
      return -5;
   }

   dwInData = 0;
   dwBytesReturned = 0;
//get main()
   if (!KernelLibIoControl(hCeoxProg, IOCTL_CEOX_GET_C_MAIN, &dwInData, sizeof(dwInData),
                              &dwCMainAddress, sizeof(dwCMainAddress), &dwBytesReturned)
       || (dwBytesReturned != 4))
   {
      printf("Error IoControl to program!\n");
      return -6;
   }
   dwInData = 0;
   dwBytesReturned = 0;
//execute main()
   if (!KernelLibIoControl(hCeoxProg, IOCTL_CEOX_CALL_C_MAIN, &dwInData, sizeof(dwInData),
                              &dwCMainAddress, sizeof(dwCMainAddress), &dwBytesReturned))
   {
      printf("Error IoControl to program!\n");
      return -6;
   }

// get pdt
   dwBytesReturned = 0;
   //JS 11.02.14
   dwInData = ulTaskNo;
   if (!KernelLibIoControl(hCeoxProg, IOCTL_CEOX_GET_PDT_ADDRESS, &dwInData, sizeof(dwInData),
                             &prcPdtAddress, sizeof(prcPdtAddress), &dwBytesReturned))
   {
      printf("Error IoControl to program!\n");
      return -6;
   }

   if(bInstallTask)
   {
// install task
      rcInstallTask.dwCMainAddress = dwCMainAddress;
      rcInstallTask.dwPdtAddress = (DWORD)prcPdtAddress;
      rcInstallTask.usNo = (USHORT) ulProgNo;
      rcInstallTask.usUsage = 0;
      rcInstallTask.usTask = (USHORT) ulTaskNo;
      rcInstallTask.usTaskType = prcPdtAddress->usFlags & 0x0007;
      rcInstallTask.usLevel = 0;
      rcInstallTask.usIrq = prcPdtAddress->usInterrupt;
      rcInstallTask.usFlags = prcPdtAddress->usFlags;
      rcInstallTask.usParSize = prcPdtAddress->usParSize;
      rcInstallTask.dwDataSize = prcPdtAddress->ulDataSize;
      if (!KernelLibIoControl(g_hCeoxIisr, IOCTL_CEOX_INSTALL_TASK, &rcInstallTask, sizeof(rcInstallTask),
                                 &rcInstallTaskOut, sizeof(rcInstallTaskOut), &dwBytesReturned))
      {
         printf("Error IoControl to CEoX kernel!\n");
         return -7;
      }
   
      if (rcInstallTaskOut.dwError != 0)
      {
         printf("CEoX internal error %X\n", rcInstallTaskOut.dwError);
         return (USHORT)rcInstallTaskOut.dwError;
      }
   }

   ULONG ulParamSize = sizeof(sExecProcParams) + sizeof(ULONG);
   pexecparams = (sExecProcParams*)malloc(ulParamSize);

   printf ("\n0: Mainproc\n");
   printf ("1: Autoinit\n");
   printf ("2, 3, ... Exported procedures\n");
   printf ("q: Quit\n");
   printf ("Enter function number: ");
   while(gets(line))
   {
      if(toupper(line[0]) == 'Q')
         break;
      if(sscanf(line, "%u", &pexecparams->usProcNo) == 1)
      {
         if(pexecparams->usProcNo < prcPdtAddress->usProcedures)
         {
            switch(pexecparams->usProcNo)
            {
            case 0:
               printf("Executing MainProc\n");
               break;
            case 1:
               printf("Executing AutoInit\n");
               break;
            default:
               printf("Executing Proc No.:%d\n", pexecparams->usProcNo);
            }
            {
               ulParamSize =sizeof(ULONG);
               pexecparams->usTaskNo = (USHORT) ulTaskNo;
            }

            if(!KernelLibIoControl(hCeoxProg, IOCTL_CEOX_EXEC_PROC, pexecparams, ulParamSize,
                              NULL, 0, &dwBytesReturned))
               printf("Error IoControl to program!\n");
         }
         else
            printf ("Invalid function number (max. %d)\n", prcPdtAddress->usProcedures - 1);
            
      }
      printf ("Enter function number: ");
   }
   return 0;
}
